#! /usr/bin/env python
###############################################################################
#
# Copyright (C) 2005  Peeter Vois
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

import os, sys, string


class execute:
	""" Runs a process
	"""
	m_pid = 0
	m_status = 0
	def __init__( self, path, frepout = 1, freperr = 2 ):
		self.m_pid = os.fork()
		if self.m_pid == 0:
			#inside child process
			if frepout != 1:
				os.close( 1 )
				os.dup2( frepout, 1 )
			if freperr != 2:
				os.close( 2 )
				os.dup2( freperr, 2 )
			os.execvp( path[0], path )
			assert 0, 'error starting program'
		#inside parent process
		
	def __del__( self ):
		if self.m_pid != 0:
			#the process is running, we have to kill it
			os.kill( self.m_pid, 9 )
			self.wait()
		#the process is closed, destroy safely

	def check( self ):
		if self.m_pid != 0:
			rpid, self.m_status = os.waitpid( self.m_pid, WNOHANG )
			if rpid == 0:
				return 0
			self.m_pid = 0
		return 1
	
	def status( self ):
		return self.m_status
							
	def wait( self ):
		if self.m_pid != 0:
			rpid, self.m_status = os.waitpid( self.m_pid, 0 )
			self.m_pid = 0
		return self.m_status


class execute_tofile( execute ):
	""" Executes subprocess, opens and sends output to file.
	"""
	
	m_out = 0
	m_err = 0
	
	def __init__ ( self, path, frepout = '/dev/null', freperr = '/dev/null' ):
		self.m_out = os.open( frepout, os.O_WRONLY | os.O_CREAT )
		self.m_err = os.open( freperr, os.O_WRONLY | os.O_CREAT )
		execute.__init__( self, path, self.m_out, self.m_err )
		
	def __del__( self ):
		os.close( self.m_out )
		os.close( self.m_err )
		execute.__del__( self )
