#! /usr/bin/env python
###############################################################################
#
# Copyright (C) 2005  Peeter Vois
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

import sys

sys.path.append('modules')

from sourceopers import *
								
"""
Tests
"""

#
# First thing to do is to define source code.
# The following line sets up entire environment for testcases.
#   -> writes source code to file
#   -> compiles it
#   -> loads tags from elf file
#
# The first line must be the compilation command
#
src = c_source( """// avr-gcc -O0 -mmcu=atmega8 <src> -o <elf>
#include <math.h>

volatile float a;
volatile float b;
volatile float c;

void test_start( void )
{
}

void test_end( void )
{
}

int main( void )
{
	test_start();
	c = a * b;
	test_end();
}
""" );

#
# The following
#   -> starts simulator
#   -> loads binary to the target
#
src.start( 'atmega8', prog = '../../simulavr-0.1.2.2/src/simulavr' )

#
# Here we define variables used for the testing algorithms.
#
var_a = 2.0
var_b = 3.0
var_c = 0.0

#
# We need to set up some breakpoints. The testing works like
# this. Target is started, then it runs to breakpoint. While
# target is stopped we upload some variables or look into 
# target memory to test the algorithm.
#
# currently c source file has always following breakpoints
#   main - the main funciton, it must be
#   exit - the trap function when main returns
#
#src.set_breakpoint( 'main' )
#src.set_breakpoint( 'exit' )
#src.set_breakpoint( 'test_start' )
#src.set_breakpoint( 'test_end' )

#
# calling wait_breakpoint automatically starts the emulator.
# if breakpoint has been reached, then the function returns
# with the breakpoint tag.
#
# following example sets variables after main function is 
# called. Then it runs the target again and waits until 
# another breakpoint is reached. If the another breakpoint
# is exit then the result is tested and test is finished.
#
src.set_breakpoint( 'main' )
bkpt = src.wait_breakpoint()
#if bkpt == 'main':
src.write_float( 'a', var_a )
src.write_float( 'b', var_b )
#else:
#	assert 0, 'unwanted breakpoint reached -> FAIL'

src.set_breakpoint( 'exit' )
bkpt = src.wait_breakpoint()

#if bkpt == 'exit':
var_c = src.read_float( 'c' )
print 'expected', var_a * var_b, 'result', var_c
#else:
#	assert 0, 'unwanted breakpoint reached -> FAIL'

#
# It is nice to clean up, close the process and do something new
#

del src

